// display_thread.c
//
//#include "display_thread.h"
#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include "test.h"

///
/// Port Connection ATmega to Display-ROW and Column Driver
/// Port A is used for the digital control
/// ADC0 is used for measuring the driver supply voltage
/// OC2A is used for driving the PWM signal to the DC/DC converter
///
/// Definition of shift register signals
/// SER : (serial DATA)
/// SCK : (shift register clock, shift with rising edge)
/// RCK : (storage register clock, copy to output register with rising edge)
/// SCL : (shift register clear input, zero clears shift register (not output reg.!))
///
///
/// ---------------------
/// ROW-Driver Connection
/// - 2x16-data bits are used, 32-bits have to be shifted in
/// - from this 32-bit the first 16-bits are driving finally the N-Channel power MOS
/// - from this 32-bit the second 16-bits are driving the P-Channel power MOS
/// - in the display only the lower 14-bits are connected each
/// - Never Never have N-channel & P-Channel bits high at the same time -> short circuit!!
///   for the driver that means that either the first or the second 16-bits have to be zero!!
///
/// Connection:
/// PA0: RCK
/// PA1: SCK
/// PA2: SER (dat)
/// PA3: SCL (shared with column driver)
/// -----------------------------------------------------------------
///
/// ---------------------
/// Column-Driver Connection
/// - 2x48-data bits are used. Both chains are filled in parallel, sharing clock and storage clock
/// - Never, Never have a high at two corresponding bits in both chains!! -> short circuit!!!
/// - easy implementation is that one of the two chains is always filled with zeros
///
/// Connection:
/// PA3: SCL (chared with row-driver)
/// PA4: RCK
/// PA5: SCK
/// PA6: SER High (2981, high driver)
/// PA7: SER low (2801, low driver)
///
/// we set a pixel, if COL is high and ROW is low
/// we clear a pixel, if ROW is high and COL is low
///

#define DRV_PORT PORTA

#define ROW_RCK (_BV(PA0))
#define ROW_SCK (_BV(PA1))
#define ROW_SER (_BV(PA2))
#define ROW_SCL (_BV(PA3))

#define COL_SCL (_BV(PA3))
#define COL_RCK (_BV(PA4))
#define COL_SCK (_BV(PA5))
#define COL_SERH (_BV(PA6))
#define COL_SERL (_BV(PA7))

//TimerControlBlock SwTimer;      // Plain vanilla timer

void clr_driver(void)
{
  DRV_PORT = 0; // clear SCL signal -> clears shift register
  DRV_PORT = ROW_RCK | COL_RCK; // copy shift register to output register
  DRV_PORT = 0;
}

void active_driver(void)
{
  DRV_PORT = ROW_RCK | COL_RCK | COL_SCL | ROW_SCL; // copy shift register to output register
  DRV_PORT = COL_SCL | ROW_SCL;  // do not clear the register
}

void set_pixline(uint8_t nr, uint8_t dat[])
///
///  set pixel on one line of the display 1x48
///  nr: nummer of the line where to set the pixels
///  dat[]: data for the line (0...13) (a one at the position sets the pixel)
///
///  to set a pixel, the row n-channel must be active and the column high-driver must be active
{
  uint8_t col;
  uint8_t rw;
  uint16_t bitcnt;
  uint8_t tmp;
  
  tmp=COL_SCL | ROW_SCL; // do not clear the shift register
  DRV_PORT = tmp;
  rw=0;
  for (col=0; col<2; col++)
  {
    for (bitcnt=1; bitcnt<=128; bitcnt*=2)
    {
      // start value for output
      // the first 16-bit are controlling the row-driver N-channel
      tmp=COL_SCL | ROW_SCL; // do not clear the shift register
      // first write data to the output port
      // column data
      if (dat[col] & bitcnt)  // column bit at position bitcnt is set
      {
        tmp |= COL_SERH;  // column high driver active
      }
      else
      {
        // do not set the corresponding bit
      }
      if (rw==nr)
      {
        tmp |= ROW_SER;  
      }
      else
      {
        // put out zero for the corresponding bit
      }
      DRV_PORT = tmp;  // data output with clock == LOW
      // clock rising edge
      asm volatile ( "nop" );
      DRV_PORT = tmp | COL_SCK | ROW_SCK;
      asm volatile ( "nop" );
      // clock falling edge
      DRV_PORT = tmp;
      asm volatile ( "nop" );
      rw++;
    }
  }
  for (col=2; col<4; col++)
  {
    for (bitcnt=1; bitcnt<=128; bitcnt*=2)
    {
      // the second 16-bit are controlling the row-driver P-channel
      // the set pixel routine should only shift out zeros here
      tmp=COL_SCL | ROW_SCL; // do not clear the shift register

      // column data
      if (dat[col] & bitcnt)  // column bit at position bitcnt is set
      {
        tmp |= COL_SERH;  // column high driver active
      }
      else
      {
        // do not set the corresponding bit
      }
      // row-data is zero!
      DRV_PORT = tmp;  // data output with clock == LOW
      asm volatile ( "nop" );      
      // clock rising edge
      DRV_PORT = tmp | COL_SCK | ROW_SCK;
      asm volatile ( "nop" );
      // clock falling edge
      DRV_PORT = tmp;
      asm volatile ( "nop" );
    }
  }
  for (col=4; col<6; col++)
  {
    // the row data has been finished, no clock at row data any more
    for (bitcnt=1; bitcnt<=128; bitcnt*=2)
    {
      // the second 16-bit are controlling the row-driver P-channel
      // the set pixel routine should only shift out zeros here
      tmp=COL_SCL | ROW_SCL; // do not clear the shift register

      // column data
      if (dat[col] & bitcnt)  // column bit at position bitcnt is set
      {
        tmp |= COL_SERH;  // column high driver active
      }
      else
      {
        // do not set the corresponding bit
      }
      // row-data is zero!
      DRV_PORT = tmp;  // data output with clock == LOW
      asm volatile ( "nop" );
      // clock rising edge
      DRV_PORT = tmp | COL_SCK;
      asm volatile ( "nop" );
      // clock falling edge
      DRV_PORT = tmp;
      asm volatile ( "nop" );
    }
  }
  // finish with writing data into the shift register
}

void clr_pixline(uint8_t nr, uint8_t dat[])
///
///  clears pixel on one line of the display 1x48
///  nr: nummer of the line where to clear the pixels
///  dat[]: data for the line (a one at the position clears the pixel)
///
{

}

void set_pixels(uint16_t row, uint8_t dat[])
///
/// write data to set pixel on the display, 14x48 Display
/// row: row-data of display
/// dat[]: column data of display
{
}

void clr_pixels()
{
}

int main(void)                 // Main runs under the AvrX Stack
{
  uint8_t dat[6];
  uint8_t nr=4;

  // Thread Initialisation
  clr_driver();

  dat[0]=0xA5;
  dat[1]=0x11;
  dat[2]=0x22;
  dat[3]=0x33;
  dat[4]=0x44;
  dat[5]=0x55;

  // the thread endless loop
  while(1)
  {
    
    PORTD |= _BV(PD6);  // LED on
    set_pixline(nr, dat);
    active_driver();
    //AvrXDelay(&SwTimer, 1);        // Delay 1 * 2ms
    clr_driver();
    PORTD &= ~_BV(PD6);  // LED off

    //AvrXDelay(&SwTimer, 100);        // Delay 100 * 2ms
  }
}



// end of file display_thread.c
